@@ -281,7 +281,6 @@ public class JavaBridge implements Runnable {
 	    Util.logMessage("socket              : " + socket);
 	    Util.logMessage("java.ext.dirs       : " + System.getProperty("java.ext.dirs"));
 	    Util.logMessage("php.java.bridge.base: " + Util.JAVABRIDGE_BASE);
-	    Util.logMessage("extra library dir   : " + Util.JAVABRIDGE_LIB);
 	    Util.logMessage("thread pool size    : " + Util.THREAD_POOL_MAX_SIZE);
 	} catch (Throwable t) {
 	    throw new RuntimeException(t);
@@ -390,13 +389,12 @@ public class JavaBridge implements Runnable {
     }
     
     void setException(Response response, Throwable e, String method, Object obj, String name, Object args[], Class params[]) {
-	if(logLevel>2) printStackTrace(e);
-
 	if (e instanceof InvocationTargetException) {
 	    Throwable t = ((InvocationTargetException)e).getTargetException();
 	    if (t!=null) e=t;
 	}
-
+	if (logLevel>3) printStackTrace(e);
+	
 	StringBuffer buf=new StringBuffer(method);
 	buf.append(" failed: ");
 	if(obj!=null) {
@@ -416,7 +414,7 @@ public class JavaBridge implements Runnable {
 	buf.append(".");
 	buf.append(" Cause: ");
 	buf.append(String.valueOf(e));
-	buf.append(" Responsible VM: ");
+	buf.append(" VM: ");
 	buf.append(Util.VM_NAME);
 	
 	lastException = new Exception(buf.toString(), e);
@@ -480,12 +478,13 @@ public class JavaBridge implements Runnable {
 
 	    Object coercedArgs[] = coerce(params=entry.getParameterTypes(selected), args, response);
 	    // If we have a logLevel of 5 or above, do very detailed invocation logging
+	    boolean hasDeclaredExceptions = selected.getExceptionTypes().length!=0;
 	    if (this.logLevel>4) {
 	        Object result = selected.newInstance(coercedArgs);
 	        logInvoke(result, name, coercedArgs);
-    	    	response.setResult(result, clazz);
+    	    	response.setResult(result, clazz, hasDeclaredExceptions);
 	    } else {
-  	    	response.setResult(selected.newInstance(coercedArgs), clazz);
+  	    	response.setResult(selected.newInstance(coercedArgs), clazz, hasDeclaredExceptions);
   	    }
 	} catch (Throwable e) {
 	    if(e instanceof InvocationTargetException) e = ((InvocationTargetException)e).getTargetException();
@@ -499,89 +498,92 @@ public class JavaBridge implements Runnable {
 		e = getUnresolvedExternalReferenceException(e, "call constructor");
 	    }
 	    setException(response, e, createInstance?"CreateInstance":"ReferenceClass", null, name, args, params);
+	    
+	    if (response.isAsync()) throw new JavaBridgeIllegalStateException ("Out of sync", e); // abort
 	}
     }
 
+    private static final Iterator EMPTY_ITERATOR = (new LinkedList()).iterator();
     //
     // Select the best match from a list of methods
     //
-    private int weight(Class param, Object arg) {
-	int w = 0;
-	if(arg!=null) {
-	 if (param.isInstance(arg)) {
- 	    for (Class c=arg.getClass(); (c=c.getSuperclass()) != null; ) {
-		if (!param.isAssignableFrom(c)) {
-		    break;
-		}
-		w+=16;		// prefer more specific arg, for
-				// example AbstractMap hashMap
-				// over Object hashMap.
-	    }
-	} else if (param == java.lang.String.class) {
-	    if (!(arg instanceof String) && !(arg instanceof PhpString))
-	        if(arg instanceof byte[])
-	            w+=32;
-	        else
-	            w+=8000; // conversion to string is always possible
-	} else if (param.isArray()) {
-	        if(arg instanceof PhpString) {
-	            Class c=param.getComponentType();
-	            if(c == byte.class) 
-	                w+=32;
-	            else
-	                w+=9999;
-	        } else if(arg.getClass() == PhpArray.class) {
-		    Iterator iterator = ((Map)arg).values().iterator();
-		    if(iterator.hasNext()) {
-			Object elem = iterator.next();
-			Class c=param.getComponentType();
-			w+=weight(c, elem);
-		    }
-		} else if(arg.getClass().isArray()) {
-		    int length = Array.getLength(arg);
-		    if(length>0) {
-			w+=weight(param.getComponentType(), Array.get(arg,0));
-		    }
-		}
-		else w+=9999;
-	    } else if ((java.util.Collection.class).isAssignableFrom(param)) {
-	    if (!(arg instanceof PhpArray))
-		w+=9999;
-	} else if (param.isPrimitive()) {
-	    Class c=param;
-	    if (arg instanceof Number) {
-		if(arg instanceof Double) {
-		    if (c==Float.TYPE) w+=1;
-		    else if (c==Double.TYPE) w+=0;
-		    else w+=256;
-		} else {
-		    if (c==Boolean.TYPE) w+=5;
-		    else if (c==Character.TYPE) w+=4;
-		    else if (c==Byte.TYPE) w+=3;
-		    else if (c==Short.TYPE) w+=2;
-		    else if (c==Integer.TYPE) w+=1;
-		    else if (c==Long.TYPE) w+=0;
-		    else w+=256;
-		}
-	    } else if (arg instanceof Boolean) {
-		if (c!=Boolean.TYPE) w+=9999;
-	    } else if (arg instanceof Character) {
-		if (c!=Character.TYPE) w+=9999;
-	    } else if ((arg instanceof String)||(arg instanceof PhpString)) {
-		    w+=64;
-	    } else {
-		w+=9999;
-	    }
-	} else if(Number.class.isAssignableFrom(param)) {
-	    if(param==Float.class || param==Double.class) {
-		if(!(arg instanceof Double)) w+=9999;
-	    } else if(!(arg instanceof PhpExactNumber)) w+=9999;
-	} else {
-	    w+=9999;
-	}
-	}
-	if(logLevel>4) logDebug("weight " + param + " " + Util.getClass(arg) + ": " +w);
-	return w;
+    private int weight(Class param, Class arg, Object phpArrayValue) {
+        int w = 0;
+        if (param.isAssignableFrom(arg)) {
+            for (Class c=arg; (c=c.getSuperclass()) != null; ) {
+                if (!param.isAssignableFrom(c)) {
+                    break;
+                }
+                w+=16;          // prefer more specific arg, for
+                                // example AbstractMap hashMap
+                                // over Object hashMap.
+            }
+        } else if (param == java.lang.String.class) {
+            if (!(String.class.isAssignableFrom(arg)) && !(PhpString.class.isAssignableFrom(arg)))
+                if(byte[].class.isAssignableFrom(arg))
+                    w+=32;
+                else
+                    w+=8000; // conversion to string is always possible
+        } else if (param.isArray()) {
+                if(PhpString.class.isAssignableFrom(arg)) {
+                    Class c=param.getComponentType();
+                    if(c == byte.class) 
+                        w+=32;
+                    else
+                        w+=9999;
+                } else if(arg == PhpArray.class) {
+                    Iterator iterator = phpArrayValue == null ? EMPTY_ITERATOR : ((Map)phpArrayValue).values().iterator();
+                    if(iterator.hasNext()) {
+                        Object elem = iterator.next();
+                        Class ptype = param.getComponentType(), atype = elem.getClass();
+                        if (ptype!=atype) {
+                            w+=(ptype==Object.class?10:8200)+weight(ptype, atype, null);
+                        }
+                    }
+                } else if(arg.isArray()) {
+                    Class ptype = param.getComponentType(), atype = arg.getComponentType();
+                    if (ptype!=atype) {
+                        w+=(ptype==Object.class?10:8200)+weight(ptype, atype, null);
+                    }
+                }
+                else w+=9999;
+            } else if ((java.util.Collection.class).isAssignableFrom(param)) {
+                if (java.util.Map.class.isAssignableFrom(arg)) w+=8100; // conversion to Collection is always possible
+                else if (!(PhpArray.class.isAssignableFrom(arg))) w+=9999;
+        } else if (param.isPrimitive()) {
+            Class c=param;
+            if (Number.class.isAssignableFrom(arg)) {
+                if(Double.class.isAssignableFrom(arg)) {
+                    if (c==Float.TYPE) w+=1;
+                    else if (c==Double.TYPE) w+=0;
+                    else w+=256;
+                } else {
+                    if (c==Boolean.TYPE) w+=5;
+                    else if (c==Character.TYPE) w+=4;
+                    else if (c==Byte.TYPE) w+=3;
+                    else if (c==Short.TYPE) w+=2;
+                    else if (c==Integer.TYPE) w+=1;
+                    else if (c==Long.TYPE) w+=0;
+                    else w+=256;
+                }
+            } else if (Boolean.class.isAssignableFrom(arg)) {
+                if (c!=Boolean.TYPE) w+=9999;
+            } else if (Character.class.isAssignableFrom(arg)) {
+                if (c!=Character.TYPE) w+=9999;
+            } else if ((String.class.isAssignableFrom(arg))||(PhpString.class.isAssignableFrom(arg))) {
+                    w+=64;
+            } else {
+                w+=9999;
+            }
+        } else if(Number.class.isAssignableFrom(param)) {
+            if(param==Float.class || param==Double.class) {
+                if(!(Double.class.isAssignableFrom(arg))) w+=9999;
+            } else if(!(PhpExactNumber.class.isAssignableFrom(arg))) w+=9999;
+        } else {
+            w+=9999;
+        }
+        if(logLevel>4) logDebug("weight " + param + " " + arg + ": " +w);
+        return w;
     }
 
     private Object select(LinkedList methods, Object args[]) {
@@ -608,8 +610,8 @@ public class JavaBridge implements Runnable {
 			arg = args[i] = proxy.getProxy(null);        
 		    }
 		}
-		
-		w+=weight(parms[i], arg);
+		if (arg!=null)
+		    w+=weight(parms[i], arg.getClass(), arg);
 	    }
 	    if (w < best) {
 		if (w == 0) {
@@ -687,12 +689,13 @@ public class JavaBridge implements Runnable {
 		result[i] = s;
 		try {
 		    if (c == Boolean.TYPE) result[i]=new Boolean(s);
-		    if (c == Byte.TYPE)    result[i]=new Byte(s);
-		    if (c == Short.TYPE)   result[i]=new Short(s);
-		    if (c == Integer.TYPE) result[i]=new Integer(s);
-		    if (c == Float.TYPE)   result[i]=new Float(s);
-		    if (c == Long.TYPE)    result[i]=new Long(s);
-		    if (c == Character.TYPE && s.length()>0)
+		    else if (c == Byte.TYPE)    result[i]=new Byte(s);
+		    else if (c == Short.TYPE)   result[i]=new Short(s);
+		    else if (c == Integer.TYPE) result[i]=new Integer(s);
+            else if (c == Float.TYPE)   result[i]=new Float(s);
+            else if (c == Double.TYPE)  result[i]=new Double(s);
+		    else if (c == Long.TYPE)    result[i]=new Long(s);
+		    else if (c == Character.TYPE && s.length()>0)
 			result[i]=new Character(s.charAt(0));
 		} catch (NumberFormatException n) {
 		    printStackTrace(n);
@@ -709,7 +712,8 @@ public class JavaBridge implements Runnable {
 		    else if (c == Byte.TYPE)    result[i]=new Byte(n.byteValue());
 		    else if (c == Short.TYPE)   result[i]=new Short(n.shortValue());
 		    else if (c == Integer.TYPE) result[i]=new Integer(n.intValue());
-		    else if (c == Float.TYPE)   result[i]=new Float(n.floatValue());
+            else if (c == Float.TYPE)   result[i]=new Float(n.floatValue());
+            else if (c == Double.TYPE)  result[i]=new Double(n.doubleValue());
 		    else if (c == Long.TYPE && !(n instanceof Long))
 			result[i]=new Long(n.longValue());
 	    	} else {
@@ -1047,14 +1051,16 @@ public class JavaBridge implements Runnable {
 		}
 		coercedArgs = coerce(params=entry.getParameterTypes(selected), args, response);
 	    } while(again);
+	    
+	    boolean hasDeclaredExceptions = selected.getExceptionTypes().length!=0;
 	    // If we have a logLevel of 5 or above, do very detailed invocation logging
 	    if (this.logLevel>4) {
 	        logInvoke(object, method, coercedArgs); 
 	        Object result = selected.invoke(object, coercedArgs);
 	        logResult(result);
-	        response.setResult(result, selected.getReturnType());
+	        response.setResult(result, selected.getReturnType(), hasDeclaredExceptions);
 	    } else {
-	        response.setResult(selected.invoke(object, coercedArgs), selected.getReturnType());	      
+	        response.setResult(selected.invoke(object, coercedArgs), selected.getReturnType(), hasDeclaredExceptions);	      
 	    }
 	} catch (Throwable e) {
 	    if(e instanceof InvocationTargetException) e = ((InvocationTargetException)e).getTargetException();
@@ -1069,7 +1075,7 @@ public class JavaBridge implements Runnable {
 	    }
 	    
             if (selected != null && e instanceof IllegalArgumentException) {
-                if (this.logLevel>1) {
+                if (this.logLevel>3) {
                     String errMsg = "\nInvoked "+method + " on "+objectDebugDescription(object)+"\n";
                     errMsg += " Expected Arguments for this Method:\n";
                     Class paramTypes[] = selected.getParameterTypes();
@@ -1090,6 +1096,8 @@ public class JavaBridge implements Runnable {
                 }
             }
 	    setException(response, e, "Invoke", object, method, args, params);
+	    
+	    if (response.isAsync()) throw new JavaBridgeIllegalStateException ("Out of sync", e); // abort
 	}
     }
 
@@ -1184,7 +1192,7 @@ public class JavaBridge implements Runnable {
 			    } else {
 			    	res=fld.get(object);
 			    }
-			    response.setResult(res, ctype);
+			    response.setResult(res, ctype, false);
 			    return;
 			}
 		    }
@@ -1214,7 +1222,8 @@ public class JavaBridge implements Runnable {
 				matches.clear();
 				break again1;
 			    }
-			    response.setResult(method.invoke(object, args), method.getReturnType());
+			    boolean hasDeclaredExceptions = method.getExceptionTypes().length!=0;
+			    response.setResult(method.invoke(object, args), method.getReturnType(), hasDeclaredExceptions);
 			    return;
 			}
 		    }
@@ -1245,7 +1254,7 @@ public class JavaBridge implements Runnable {
 			    } else {
 				res = fld.get(object);
 			    }
-			    response.setResult(res, ctype);
+			    response.setResult(res, ctype, false);
 			    return;
 			}
 		    }
@@ -1265,6 +1274,8 @@ public class JavaBridge implements Runnable {
 		e = getUnresolvedExternalReferenceException(e, "invoke a property");
 	    }
 	    setException(response, e, set?"SetProperty":"GetProperty", object, prop, args, params);
+	    
+	    if (response.isAsync()) throw new JavaBridgeIllegalStateException ("Out of sync.", e); // abort
 	}
     }
 
@@ -1519,9 +1530,9 @@ public class JavaBridge implements Runnable {
      * @param claz The class or an instance of a class
      * @return true if ob is an instance of class, false otherwise.
      */
-    public static boolean InstanceOf(Object ob, Object claz) {
+    public boolean InstanceOf(Object ob, Object claz) {
 	Class clazz = Util.getClass(claz);
-	return clazz.isInstance(ob);
+	return clazz.isInstance(castToBoolean(ob));
     }
 
     /**
@@ -1530,23 +1541,18 @@ public class JavaBridge implements Runnable {
      * @return A string representation.
      */
     public String ObjectToString(Object ob) {
-	    StringBuffer buf = new StringBuffer("[");
-	try {
-	    Util.appendObject(ob, buf);
-	} catch (Request.AbortException sub) {
-	    throw sub;
-	} catch (Exception t) {
-		    Util.printStackTrace(t);
-		    buf.append("[Exception in toString(): ");
-		    buf.append(t);
-		    if(t.getCause()!=null) {
-		      buf.append(", Cause: ");
-		      buf.append(t.getCause());
-		    }
-		    buf.append("]");
-		}
-	    buf.append("]");
-	    return (String)castToString(buf.toString());
+	if (ob==null && !options.preferValues()) ob = Request.PHPNULL;
+	return (String)castToString(Util.stringValueOf(ob));
+    }
+
+    /**
+     * Returns a string representation of the object
+     * @param ob The object
+     * @return A string representation.
+     */
+    public String ObjectToString(Boolean ob) {
+	if (ob==null) return ObjectToString((Object) null);   
+	return (String)castToString(ob.booleanValue()?"1":"");
     }
 
     /**
@@ -1555,7 +1561,8 @@ public class JavaBridge implements Runnable {
      * @return A string representation.
      */
     public String ObjectToString(String ob) {
-	    return (String)castToString(ob);
+	if (ob==null) return ObjectToString((Object) null);   
+	return (String)castToString(ob);
     }
 
     /**
@@ -1564,11 +1571,12 @@ public class JavaBridge implements Runnable {
      * @return A string representation.
      */
     public String ObjectToString(byte[] ob) {
-	   try {
-	        return new String ((byte[])castToString(ob), options.getEncoding());
-	    } catch (UnsupportedEncodingException e) {
-		return new String ((byte[])castToString(ob));
-	    }
+	if (ob==null) return ObjectToString((Object) null);   
+	try {
+	    return new String ((byte[])castToString(ob), options.getEncoding());
+    	} catch (UnsupportedEncodingException e) {
+    	    return new String ((byte[])castToString(ob));
+    	}
     }
  
     /**
@@ -1614,12 +1622,13 @@ public class JavaBridge implements Runnable {
      * the servlet or jsp.
      * @param name The session name, if any
      * @param clientIsNew true, if the client wants a new session
-     * @param timeout session timeout in seconds
+     * @param timeout session timeout in seconds. If timeout is <= 0, the session will never expire
      * @return The session context.
      * @throws Exception 
      * @see php.java.bridge.ISession
      */
     public ISession getSession(String name, boolean clientIsNew, int timeout) throws Exception {
+        if (timeout == 0) timeout = -1;
 	if(sessionCache!=null) return sessionCache;
 	try {
 	ISession session= sessionFactory.getSession(name, clientIsNew, timeout);
@@ -1749,7 +1758,7 @@ public class JavaBridge implements Runnable {
 	ISession session = defaultSessionFactory.getSessionInternal(false, timeout);
 	Object obj = session.get(serialID);
 	if(obj==null) throw new IllegalArgumentException("Session serialID " +  serialID + " expired.");
-	return globalRef.append(obj);
+	return globalRef.append(castToExact(obj));
     }
     private static int counter=0;
     private static synchronized int getSerialID() {
@@ -2028,7 +2037,6 @@ public class JavaBridge implements Runnable {
      * @return The response object
      */
     Response createResponse() {
-	if(options.passContext()) return new ClassicResponse(this);
 	return new Response(this);
     }
     
