@@ -1,12 +1,7 @@
 #!/usr/bin/php
 
 <?php
-if (!extension_loaded('java')) {
-  if (!(require_once("http://127.0.0.1:8080/JavaBridge/java/Java.inc"))) {
-    echo "java extension not installed.";
-    exit(2);
-  }
-}
+require_once ("java/Java.inc");
 
 // the following tests if a class "NoClassDefFound" which contains a
 // constructor and a method which internally reference an external
